package com.oqpo.api.service.inoutmng;


import com.oqpo.api.entity.inoutmng.*;
import com.oqpo.api.entity.loanmng.LoanMgntBaseInfoEntity;
import com.oqpo.api.entity.oper.FileEntity;
import com.oqpo.api.entity.pomng.PchOdrBaseInfoEntity;
import com.oqpo.api.entity.pomng.PchOdrDtlPtclEntity;
import com.oqpo.api.entity.rtnmng.RtnOdrBaseInfoEntity;
import com.oqpo.api.entity.rtnmng.RtnOdrDtlPtclEntity;
import com.oqpo.api.entity.rtnmng.RtnProcInfoPtclEntity;
import com.oqpo.api.entity.stinfo.BrandEntity;
import com.oqpo.api.entity.stinfo.SpplyEntity;
import com.oqpo.api.entity.stinfo.StoreBaseInfoEntity;
import com.oqpo.api.entity.stockmng.StckMgntBaseInfoEntity;
import com.oqpo.api.entity.stockmng.StckMgntDtlHistEntity;
import com.oqpo.api.entity.stockmng.WhsProcInfoPtclEntity;
import com.oqpo.api.enums.ChgDvsn;
import com.oqpo.api.enums.InOutCd;
import com.oqpo.api.enums.SystemMessageCode;
import com.oqpo.api.exception.GlobalException;
import com.oqpo.api.mapper.FileMapper;
import com.oqpo.api.mapper.inoutmng.ColOdrMapper;
import com.oqpo.api.mapper.inoutmng.ShmtOdrMapper;
import com.oqpo.api.mapper.loanmng.LoanStateMapper;
import com.oqpo.api.mapper.pomng.PchOrdMapper;
import com.oqpo.api.mapper.rtnmng.RtnOdrMapper;
import com.oqpo.api.mapper.stinfo.BrandMapper;
import com.oqpo.api.mapper.stinfo.StoreBaseInfoMapper;
import com.oqpo.api.mapper.stinfo.SupplyMapper;
import com.oqpo.api.mapper.stockmng.StockMngMapper;
import com.oqpo.api.service.CommonService;
import com.oqpo.api.service.pomng.PchOrdService;
import com.oqpo.api.service.rtnmng.RtnOdrService;
import com.oqpo.api.web.dto.request.GridRequest;
import com.oqpo.api.web.dto.request.inoutmng.*;
import com.oqpo.api.web.dto.response.GridResponse;
import com.oqpo.api.web.dto.response.inoutmng.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Service
@Slf4j
public class ColOdrService extends CommonService {
    @Autowired
    private ColOdrMapper colOdrMapper;

    @Autowired
    private RtnOdrMapper rtnOdrMapper;

    @Autowired
    private RtnOdrService rtnOdrService;


    @Autowired
    private BrandMapper brandMapper;

    @Autowired
    private SupplyMapper supplyMapper;

    @Autowired
    private StoreBaseInfoMapper storeBaseInfoMapper;

    @Autowired
    private LoanStateMapper loanStateMapper;

    @Autowired
    private FileMapper fileMapper;

    /*
      수거 대상 그리드 리스트 조회
     */
    public GridResponse selectShmtTargetOdrGridList(String sBrandId, String fromDt, String toDt, String sWhsId, String sRtnWhsId, String sItemNm, GridRequest gridRequest) throws Exception {
        int gridPage = gridRequest.getGridPage();
        int gridSize = gridRequest.getGridSize();

        fromDt = fromDt == null ? null : fromDt.replace(".", "");
        toDt = toDt == null ? null : toDt.replace(".", "");

        int gridRecords = colOdrMapper.selectColTargetRtnGridCnt(sBrandId, fromDt, toDt, sWhsId, sRtnWhsId, sItemNm);
        int gridTotal = fnCalculateGridTotal(gridSize, gridRecords);
        List<RtnOdrBaseInfoEntity> entities = colOdrMapper.selectColTargetRtnGridList(sBrandId, fromDt, toDt, sWhsId,sRtnWhsId, sItemNm, gridRequest);
        List<Object> gridRows = entities.stream()
                .map(m -> ColTragetOdrSearchListResponse.builder()
                        .viewCd("R")
                        .brandId(m.getBrandId() == null ? "" : m.getBrandId())
                        .brandNm(m.getBrandNm() == null ? "" : m.getBrandNm())
                        .rtnPodrUnqNo(m.getRtnPodrUnqNo() == null ? "" : m.getRtnPodrUnqNo())
                        .rtnPodrDtlNo(m.getRtnOdrDtlInfo() == null ? "" : m.getRtnOdrDtlInfo().getRtnPodrDtlNo() == null ? "" : m.getRtnOdrDtlInfo().getRtnPodrDtlNo())
                        .rtnReqUnqNo(m.getRtnOdrDtlInfo() == null ? "" : m.getRtnOdrDtlInfo().getRtnReqUnqNo() == null ? "" : m.getRtnOdrDtlInfo().getRtnReqUnqNo())
                        .rtnReqDtlNo(m.getRtnOdrDtlInfo() == null ? "" : m.getRtnOdrDtlInfo().getRtnReqDtlNo() == null ? "" : m.getRtnOdrDtlInfo().getRtnReqDtlNo())
                        .itemId(m.getRtnOdrDtlInfo() == null ? "" : m.getRtnOdrDtlInfo().getItemId() == null ? "" : m.getRtnOdrDtlInfo().getItemId())
                        .itemNm(m.getRtnOdrDtlInfo() == null ? "" : m.getRtnOdrDtlInfo().getItemNm() == null ? "" : m.getRtnOdrDtlInfo().getItemNm())
                        .rtnWhsId(m.getRtnWhsId() == null ? "" : m.getRtnWhsId())
                        .rtnLocation(m.getRtnLocation() == null ? "" : m.getRtnLocation())
                        .rntWhsNm(m.getRtnWhsNm() == null ? "" : m.getRtnWhsNm())
                        .whsId(m.getWhsId() == null ? "" : m.getWhsId())
                        .location(m.getLocation() == null ? "" : m.getLocation())
                        .whsNm(m.getWhsNm() == null ? "" : m.getWhsNm())
                        .unitAmt(m.getRtnOdrDtlInfo() == null ? 0 : m.getRtnOdrDtlInfo().getUnitAmt() == null ? 0 : m.getRtnOdrDtlInfo().getUnitAmt())
                        .rtnReqQty(m.getRtnOdrDtlInfo() == null ? 0 : m.getRtnOdrDtlInfo().getRtnReqQty() == null ? 0 : m.getRtnOdrDtlInfo().getRtnReqQty())
                        .rtnQty(m.getRtnOdrDtlInfo() == null ? 0 : m.getRtnOdrDtlInfo().getRtnQty() == null ? 0 : m.getRtnOdrDtlInfo().getRtnQty())
                        .rtnAmt(m.getRtnOdrDtlInfo() == null ? 0 : m.getRtnOdrDtlInfo().getRtnAmt() == null ? 0 : m.getRtnOdrDtlInfo().getRtnAmt())
                        .colReqDt(m.getColReqDt() == null ? "" : m.getColReqDt())
                        .storeId(m.getStoreId() == null ? "" : m.getStoreId())
                        .build())
                .collect(Collectors.toList());
        return GridResponse.toDTO(gridPage, gridTotal, gridRecords, gridRows);
    }

    /*
      수거지시 저장
*/
    @Transactional
    public void instColOdr(String userId, InstColOdrRequest instColOdrRequest) throws Exception {
        try {

            List<RtnOdrDtlPtclEntity> gridInsertData = instColOdrRequest.toCodeEntities(instColOdrRequest.getGridInsertData());


            List<RtnOdrBaseInfoEntity> jobList = colOdrMapper.selectPickJobDataList(instColOdrRequest.getBrandId(), gridInsertData);

            String whsId = "";
            String location = "";
            String rtnWhsId = "";
            String rtnLocation = "";
            int itemQty = 0;
            long colTotalAmt = 0;
            int colQty = 0;
            String colCmdUnqNo = "";
            String colPickUnqNo = "";

            ColOdrBaseEntity colOdrBaseEntity = new ColOdrBaseEntity();
            List<ColOdrDtlPtclEntity> colDtlList = new ArrayList<ColOdrDtlPtclEntity>();

            PickInfoMgntEntity pickInfoMgntEntity = new PickInfoMgntEntity();
            List<PickDtlPtclEntity> pickDtlList = new ArrayList<PickDtlPtclEntity>();

            for (int i = 0; i < jobList.size(); i++) {
                RtnOdrBaseInfoEntity entity = jobList.get(i);

                for (int j = 0; j < gridInsertData.size(); j++) {
                    RtnOdrDtlPtclEntity reqEntity = gridInsertData.get(j);

                    if (entity.getRtnPodrUnqNo().equals(reqEntity.getRtnPodrUnqNo()) && entity.getRtnOdrDtlInfo().getRtnPodrDtlNo().equals(reqEntity.getRtnPodrDtlNo())) {
                        if (reqEntity.getColQty() > entity.getRtnOdrDtlInfo().getRtnOdrQty()) {
                            // 반품발주수량보다 수거수량이 많을수 없습니다.
                            throw new GlobalException(SystemMessageCode.ERR_POPICKQTY);
                        }

                        colQty = reqEntity.getColQty();

                        break;
                    }
                }

                if (rtnWhsId.equals(entity.getRtnWhsId()) && rtnLocation.equals(entity.getRtnLocation()) && whsId.equals(entity.getWhsId()) && location.equals(entity.getLocation())) {
                    // 추가 작업
                    itemQty = itemQty + 1;
                    colTotalAmt = colTotalAmt + (entity.getRtnOdrDtlInfo().getUnitAmt() * colQty);
                    colOdrBaseEntity.setItemQty(itemQty);
                    colOdrBaseEntity.setColTotalAmt(colTotalAmt);
                    pickInfoMgntEntity.setColTotAmt(colTotalAmt);

                    //수거서 상세 정보
                    int dtlNo = itemQty * 10;

                    ColOdrDtlPtclEntity colDtlEntity = new ColOdrDtlPtclEntity();
                    colDtlEntity.setColCmdUnqNo(colCmdUnqNo);
                    colDtlEntity.setColCmdDtlNo(Integer.toString(dtlNo));
                    colDtlEntity.setItemId(entity.getRtnOdrDtlInfo().getItemId());
                    colDtlEntity.setItemNm(entity.getRtnOdrDtlInfo().getItemNm());
                    colDtlEntity.setUnitAmt(entity.getRtnOdrDtlInfo().getUnitAmt());
                    colDtlEntity.setColQty(colQty);
                    colDtlEntity.setColAmt(colQty * entity.getRtnOdrDtlInfo().getUnitAmt());
                    colDtlEntity.setRtnQty(entity.getRtnOdrDtlInfo().getRtnOdrQty());
                    colDtlEntity.setRtnAmt(entity.getRtnOdrDtlInfo().getRtnOdrAmt());
                    colDtlEntity.setRtnRsnDvsn(entity.getRtnOdrDtlInfo().getRtnRsnDvsn());
                    colDtlEntity.setRtnRsn(entity.getRtnOdrDtlInfo().getRtnRsn());
                    colDtlEntity.setRtnPodrUnqNo(entity.getRtnPodrUnqNo());
                    colDtlEntity.setRtnPodrDtlNo(entity.getRtnOdrDtlInfo().getRtnPodrDtlNo());
                    colDtlEntity.setRtnReqUnqNo(entity.getRtnOdrDtlInfo().getRtnReqUnqNo());
                    colDtlEntity.setRtnReqDtlNo(entity.getRtnOdrDtlInfo().getRtnReqDtlNo());

                    colDtlList.add(colDtlEntity);

                    PickDtlPtclEntity pickDtlEntity = new PickDtlPtclEntity();
                    pickDtlEntity.setColPickUnqNo(colPickUnqNo);

                    pickDtlEntity.setColPickDtlNo(Integer.toString(dtlNo));
                    pickDtlEntity.setRtnPodrUnqNo(entity.getRtnPodrUnqNo());
                    pickDtlEntity.setRtnPodrDtlNo(entity.getRtnOdrDtlInfo().getRtnPodrDtlNo());
                    pickDtlEntity.setRtnReqUnqNo(entity.getRtnOdrDtlInfo().getRtnReqUnqNo());
                    pickDtlEntity.setRtnReqDtlNo(entity.getRtnOdrDtlInfo().getRtnReqDtlNo());
                    pickDtlEntity.setColCmdUnqNo(colCmdUnqNo);
                    pickDtlEntity.setColCmdDtlNo(Integer.toString(dtlNo));
                    pickDtlEntity.setItemId(entity.getRtnOdrDtlInfo().getItemId());
                    pickDtlEntity.setItemNm(entity.getRtnOdrDtlInfo().getItemNm());
                    pickDtlEntity.setUnit(entity.getRtnOdrDtlInfo().getUnit());
                    pickDtlEntity.setUnitAmt(entity.getRtnOdrDtlInfo().getUnitAmt());
                    pickDtlEntity.setRodrQty(entity.getRtnOdrDtlInfo().getRtnOdrQty());
                    pickDtlEntity.setColQty(colQty);
                    pickDtlEntity.setColAmt(colQty * entity.getRtnOdrDtlInfo().getUnitAmt());

                    pickDtlList.add(pickDtlEntity);

                } else {
                    // 신규로 출하건 생성함  //select fn_get_dealno(16, '') as col_cmd_unq_no
                    if (colDtlList.size() > 0) {

                        colOdrMapper.insertColOdrBase(userId,colOdrBaseEntity);
                        colOdrMapper.insertColOdrDtl(userId,colDtlList);
                        colOdrMapper.insertPickMgntBase(userId,pickInfoMgntEntity);
                        colOdrMapper.insertPickDtlPtcl(userId,pickDtlList);

                        itemQty = 0;
                        colTotalAmt = 0;
                    }
                    itemQty = 1;
                    colTotalAmt = entity.getRtnOdrDtlInfo().getUnitAmt() * colQty;

                    whsId = entity.getWhsId();
                    location = entity.getLocation();
                    rtnWhsId = entity.getRtnWhsId();
                    rtnLocation = entity.getRtnLocation();

                    colCmdUnqNo = fnGetDealNo(20, "");  // 출하지시번호
                    colPickUnqNo = fnGetDealNo(15, ""); // 납품서번호

                    // 수거지시
                    colOdrBaseEntity = new ColOdrBaseEntity();
                    colDtlList = new ArrayList<ColOdrDtlPtclEntity>();
                    ColOdrDtlPtclEntity colDtlEntity = new ColOdrDtlPtclEntity();

                    //납품서정보
                    pickInfoMgntEntity= new PickInfoMgntEntity();
                    pickDtlList = new ArrayList<PickDtlPtclEntity>();
                    PickDtlPtclEntity pickDtlEntity = new PickDtlPtclEntity();

                    colOdrBaseEntity.setColCmdUnqNo(colCmdUnqNo);
                    colOdrBaseEntity.setBrandId(entity.getBrandId());
                    colOdrBaseEntity.setWhsId(entity.getWhsId());
                    colOdrBaseEntity.setLocation(entity.getLocation());
                    colOdrBaseEntity.setStoreId(entity.getStoreId());
                    colOdrBaseEntity.setColPickUnqNo(colPickUnqNo);
                    colOdrBaseEntity.setItemQty(itemQty);
                    colOdrBaseEntity.setColTotalAmt(colTotalAmt);
                    colOdrBaseEntity.setColReqDt(instColOdrRequest.getColSchDt().replaceAll("//.",""));
                    colOdrBaseEntity.setColDvsn("C001");
                    colOdrBaseEntity.setColCmdStCd("COL1");
                    colOdrBaseEntity.setColReqMgrNm(entity.getRtnMgrNm());
                    colOdrBaseEntity.setColReqMgrId(entity.getRtnMgrId());
                    colOdrBaseEntity.setColDt(null);
                    colOdrBaseEntity.setColCnfmNm(null);
                    colOdrBaseEntity.setColCnfmId(null);
                    colOdrBaseEntity.setColMgrNm(instColOdrRequest.getPickMgrNm());
                    colOdrBaseEntity.setColMgrTelNo(instColOdrRequest.getPickMgrTelNo());
                    colOdrBaseEntity.setColMgrEmal(null);
                    colOdrBaseEntity.setColMgrMblNo(null);
                    colOdrBaseEntity.setRtnWhsId(entity.getRtnWhsId());
                    colOdrBaseEntity.setRtnLocation(entity.getRtnLocation());
                    colOdrBaseEntity.setSpplyId(entity.getSpplyId());
                    colOdrBaseEntity.setSpplyNm(entity.getSpplyNm());

                    colDtlEntity.setColCmdUnqNo(colCmdUnqNo);
                    colDtlEntity.setColCmdDtlNo("10");
                    colDtlEntity.setItemId(entity.getRtnOdrDtlInfo().getItemId());
                    colDtlEntity.setItemNm(entity.getRtnOdrDtlInfo().getItemNm());
                    colDtlEntity.setUnitAmt(entity.getRtnOdrDtlInfo().getUnitAmt());
                    colDtlEntity.setColQty(colQty);
                    colDtlEntity.setColAmt(colQty * entity.getRtnOdrDtlInfo().getUnitAmt());
                    colDtlEntity.setRtnQty(entity.getRtnOdrDtlInfo().getRtnOdrQty());
                    colDtlEntity.setRtnAmt(entity.getRtnOdrDtlInfo().getRtnOdrAmt());
                    colDtlEntity.setRtnRsnDvsn(entity.getRtnOdrDtlInfo().getRtnRsnDvsn());
                    colDtlEntity.setRtnRsn(entity.getRtnOdrDtlInfo().getRtnRsn());
                    colDtlEntity.setRtnPodrUnqNo(entity.getRtnPodrUnqNo());
                    colDtlEntity.setRtnPodrDtlNo(entity.getRtnOdrDtlInfo().getRtnPodrDtlNo());
                    colDtlEntity.setRtnReqUnqNo(entity.getRtnOdrDtlInfo().getRtnReqUnqNo());
                    colDtlEntity.setRtnReqDtlNo(entity.getRtnOdrDtlInfo().getRtnReqDtlNo());


                    pickInfoMgntEntity.setColPickUnqNo(colPickUnqNo);
                    pickInfoMgntEntity.setSpplyId(entity.getSpplyId());
                    pickInfoMgntEntity.setSpplyNm(entity.getSpplyNm());
                    pickInfoMgntEntity.setStoreId(entity.getStoreId());
                    pickInfoMgntEntity.setColTotAmt(colTotalAmt);
                    pickInfoMgntEntity.setPickMgrNm(instColOdrRequest.getPickMgrNm());
                    pickInfoMgntEntity.setPickMgrTelNo(instColOdrRequest.getPickMgrTelNo().replaceAll("-",""));
                    pickInfoMgntEntity.setPndrTelNo(entity.getPickTelNo().replaceAll("-",""));
                    pickInfoMgntEntity.setPickStCd("PICK0");
                    pickInfoMgntEntity.setColReqDt(instColOdrRequest.getColSchDt().replaceAll("-",""));
                    pickInfoMgntEntity.setColSchDt(instColOdrRequest.getColSchDt().replaceAll("-",""));
                    pickInfoMgntEntity.setColCmpltDt(null);
                    pickInfoMgntEntity.setWhsId(entity.getWhsId());
                    pickInfoMgntEntity.setLocation(entity.getLocation());
                    pickInfoMgntEntity.setColDvsn(entity.getColDvsn());
                    pickInfoMgntEntity.setWhsDvsn(entity.getColDvsn() == "C001" ? "W02" : "W01");  // 본사엣 수거면 매장,,, 공급사에서 수거면 본사
                    pickInfoMgntEntity.setBrandId(entity.getBrandId());
                    pickInfoMgntEntity.setColDesc(null);


                    pickDtlEntity.setColPickUnqNo(colPickUnqNo);

                    pickDtlEntity.setColPickDtlNo("10");
                    pickDtlEntity.setRtnPodrUnqNo(entity.getRtnPodrUnqNo());
                    pickDtlEntity.setRtnPodrDtlNo(entity.getRtnOdrDtlInfo().getRtnPodrDtlNo());
                    pickDtlEntity.setRtnReqUnqNo(entity.getRtnOdrDtlInfo().getRtnReqUnqNo());
                    pickDtlEntity.setRtnReqDtlNo(entity.getRtnOdrDtlInfo().getRtnReqDtlNo());
                    pickDtlEntity.setColCmdUnqNo(colCmdUnqNo);
                    pickDtlEntity.setColCmdDtlNo("10");
                    pickDtlEntity.setItemId(entity.getRtnOdrDtlInfo().getItemId());
                    pickDtlEntity.setItemNm(entity.getRtnOdrDtlInfo().getItemNm());
                    pickDtlEntity.setUnit(entity.getRtnOdrDtlInfo().getUnit());
                    pickDtlEntity.setUnitAmt(entity.getRtnOdrDtlInfo().getUnitAmt());
                    pickDtlEntity.setRodrQty(entity.getRtnOdrDtlInfo().getRtnOdrQty());
                    pickDtlEntity.setColQty(colQty);
                    pickDtlEntity.setColAmt(colQty * entity.getRtnOdrDtlInfo().getUnitAmt());

                    pickDtlList.add(pickDtlEntity);
                }

                // 마지막 부분일 경우 
                if (i == jobList.size() - 1) {
                    if (colDtlList.size() > 0) {
                        colOdrMapper.insertColOdrBase(userId,colOdrBaseEntity);
                        colOdrMapper.insertColOdrDtl(userId,colDtlList);
                        colOdrMapper.insertPickMgntBase(userId,pickInfoMgntEntity);
                        colOdrMapper.insertPickDtlPtcl(userId,pickDtlList);
                    }
                }

                // 수거상태 변경처리
                // 1. 수거상세정보 상태 없데이트 한후  pchOrdMapper

                for (int poi = 0; poi < gridInsertData.size(); poi++) {
                    gridInsertData.get(poi).setRodrDtlStCd("ROD3");
                }
                rtnOdrMapper.updateChangeMultiRodrDtlStCd(userId, gridInsertData);

                // 반품발주 헤더 상태 처리  updateChgMultiPodrStCd
                String rtnPodrUnqNo = "";
                for (int k = 0; k < gridInsertData.size(); k++) {
                    if (!rtnPodrUnqNo.equals(gridInsertData.get(k).getRtnPodrUnqNo())) {
                        rtnPodrUnqNo = gridInsertData.get(k).getRtnPodrUnqNo();
                        String rtnOdrStCd = rtnOdrService.selectRtnOdrDtlStatusList(rtnPodrUnqNo);
                        rtnOdrMapper.updateChgRodrStCd(userId, rtnPodrUnqNo, rtnOdrStCd);
                    }
                }
            }


        } catch (GlobalException e) {
            //e.getStackTrace();
            e.printStackTrace();
            throw new GlobalException(e.getSystemMessageCode());
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
    }

    /* 수거현황 */
    public GridResponse selectColSearchGridList(String sBrandId, String fromDt, String toDt, String sColDvsn, String sWhsId, String sRtnWhsId, String sColCmdDtlStCd,  String sItemNm, GridRequest gridRequest) throws Exception {
        int gridPage = gridRequest.getGridPage();
        int gridSize = gridRequest.getGridSize();

        fromDt = fromDt == null ? null : fromDt.replace(".", "");
        toDt = toDt == null ? null : toDt.replace(".", "");

        int gridRecords = colOdrMapper.selectColGridCnt(sBrandId, fromDt, toDt, sColDvsn, sWhsId,  sRtnWhsId, sColCmdDtlStCd, sItemNm);
        int gridTotal = fnCalculateGridTotal(gridSize, gridRecords);
        List<ColOdrBaseEntity> entities = colOdrMapper.selectColGridList(sBrandId, fromDt, toDt, sColDvsn, sWhsId,  sRtnWhsId, sColCmdDtlStCd, sItemNm, gridRequest);
        List<Object> gridRows = entities.stream()
                .map(m -> ColSearchListResponse.builder()
                        .viewCd("R")
                        .brandId(m.getBrandId() == null ? "" : m.getBrandId())
                        .brandNm(m.getBrandNm() == null ? "" : m.getBrandNm())
                        .colCmdUnqNo(m.getColCmdUnqNo() == null ? "" : m.getColCmdUnqNo())
                        .colCmdDtlNo(m.getColDtlInfo() == null ? "" : m.getColDtlInfo().getColCmdDtlNo() == null ? "" :   m.getColDtlInfo().getColCmdDtlNo())
                        .rtnPodrUnqNo(m.getColDtlInfo() == null ? "" : m.getColDtlInfo().getRtnPodrUnqNo() == null ? "" :   m.getColDtlInfo().getRtnPodrUnqNo())
                        .rtnPodrDtlNo(m.getColDtlInfo() == null ? "" : m.getColDtlInfo().getRtnPodrDtlNo() == null ? "" :   m.getColDtlInfo().getRtnPodrDtlNo())
                        .colCmdDtlStCd(m.getColDtlInfo() == null ? "" : m.getColDtlInfo().getColCmdDtlStCd() == null ? "" :   m.getColDtlInfo().getColCmdDtlStCd())
                        .colCmdDtlStNm(m.getColDtlInfo() == null ? "" : m.getColDtlInfo().getColCmdDtlStNm() == null ? "" :   m.getColDtlInfo().getColCmdDtlStNm())
                        .itemId(m.getColDtlInfo() == null ? "" : m.getColDtlInfo().getItemId() == null ? "" :   m.getColDtlInfo().getItemId())
                        .itemNm(m.getColDtlInfo() == null ? "" : m.getColDtlInfo().getItemNm() == null ? "" :   m.getColDtlInfo().getItemNm())
                        .rtnWhsId(m.getRtnWhsId() == null ? "" : m.getRtnWhsId())
                        .rtnLocation(m.getRtnLocation() == null ? "" : m.getRtnLocation())
                        .rtnWhsNm(m.getRtnWhsNm() == null ? "" : m.getRtnWhsNm())
                        .colDvsn(m.getColDvsn() == null ? "" : m.getColDvsn())
                        .colDvsnNm(m.getColDvsnNm() == null ? "" : m.getColDvsnNm())
                        .whsId(m.getWhsId() == null ? "" : m.getWhsId())
                        .whsNm(m.getWhsNm() == null ? "" : m.getWhsNm())
                        .rtnQty(m.getColDtlInfo() == null ? 0 : m.getColDtlInfo().getRtnQty() == null ? 0 :   m.getColDtlInfo().getRtnQty())
                        .colQty(m.getColDtlInfo() == null ? 0 : m.getColDtlInfo().getColQty() == null ? 0 :   m.getColDtlInfo().getColQty())
                        .colAmt(m.getColDtlInfo() == null ? 0 : m.getColDtlInfo().getColAmt() == null ? 0 :   m.getColDtlInfo().getColAmt())
                        .colDt(m.getColDt() == null ? "" : m.getColDt())
                        .colCnfmNm(m.getColCnfmNm() == null ? "" : m.getColCnfmNm())
                        .colPickUnqNo(m.getColPickUnqNo() == null ? "" : m.getColPickUnqNo())
                        .build())
                .collect(Collectors.toList());
        return GridResponse.toDTO(gridPage, gridTotal, gridRecords, gridRows);
    }

    /* 수거서 정보 */
    public ColInfoResponse selectPickInfo(String brandId, String colPickUnqNo) throws Exception {

        PickInfoMgntEntity entity = colOdrMapper.selectPickInfo(brandId, colPickUnqNo);
        List<PickDtlPtclEntity> dtlEntity = colOdrMapper.selectPickDtlPtcl(brandId, colPickUnqNo);


        // 공급사
        // 수거처 정보
        BrandEntity brandEntity = brandMapper.selectBrandInfo(brandId);

        if ("C001".equals(entity.getColDvsn())) {   // 본사수거
            //브랜드정보 수거
            entity.setColBsnsRegno(brandEntity.getBsnsRegNo());
            entity.setColNm(brandEntity.getBrandNm());
            entity.setColOwnerNm(brandEntity.getOwnerNm());
            entity.setColAddr(brandEntity.getAddr1() + " " + brandEntity.getAddr2());

        } else {
            //공급사 수거
            SpplyEntity dlv_spplyEntity = supplyMapper.selectSpplyInfo(brandId, entity.getSpplyId());
            entity.setColBsnsRegno(dlv_spplyEntity.getBsnsRegNo());
            entity.setColNm(dlv_spplyEntity.getSpplyNm());
            entity.setColOwnerNm(dlv_spplyEntity.getOwnerNm());
            entity.setColAddr(dlv_spplyEntity.getAddr1() + " " + dlv_spplyEntity.getAddr2());
        }


        //반품처 정보  -- 수거장소가 매장
        if ("W01".equals(entity.getWhsDvsn())) {
            //납품창고 본사
            entity.setRtnBsnsRegno(brandEntity.getBsnsRegNo());
            entity.setRtnNm(brandEntity.getBrandNm());
            entity.setRtnOwnerNm(brandEntity.getOwnerNm());
            entity.setRtnAddr(brandEntity.getAddr1() + " " + brandEntity.getAddr2());
        } else {
            //납품창고 매장
            StoreBaseInfoEntity whs_storeBaseInfoEntity = storeBaseInfoMapper.selectStoreInfo(brandId, entity.getStoreId());
            entity.setRtnBsnsRegno(whs_storeBaseInfoEntity.getBsnsRegNo());
            entity.setRtnNm(whs_storeBaseInfoEntity.getStoreNm());
            entity.setRtnOwnerNm(whs_storeBaseInfoEntity.getOwnerNm());
            entity.setRtnAddr(whs_storeBaseInfoEntity.getAddr1() + " " + whs_storeBaseInfoEntity.getAddr2());
        }

        // 브랜드 로고 brand_logo_file_no
        List<FileEntity> logFileList = null;
        if (!"".equals(brandEntity.getBrandLogoFileNo())) {
            logFileList = fileMapper.selectFileList(brandEntity.getBrandLogoFileNo());
        } else {
            logFileList = new ArrayList<FileEntity>();
        }

        return ColInfoResponse.toDTO(entity, dtlEntity, logFileList);
    }


    /*
     공급사 수거대상 그리드 리스트 조회
*/
    public GridResponse selectSpplyTargetRtnGridList(String sBrandId, String fromDt, String toDt, String sSpplyId, String sRtnWhsId, String sItemNm, GridRequest gridRequest) throws Exception {
        int gridPage = gridRequest.getGridPage();
        int gridSize = gridRequest.getGridSize();

        fromDt = fromDt == null ? null : fromDt.replace(".", "");
        toDt = toDt == null ? null : toDt.replace(".", "");

        int gridRecords = colOdrMapper.selectColSpplyTargetRtnGridCnt(sBrandId,fromDt, toDt,sSpplyId,  sRtnWhsId, sItemNm);
        int gridTotal = fnCalculateGridTotal(gridSize, gridRecords);
        List<RtnOdrBaseInfoEntity> entities = colOdrMapper.selectColSpplyTargetRtnGridList(sBrandId,fromDt, toDt,sSpplyId,  sRtnWhsId, sItemNm, gridRequest);
        List<Object> gridRows = entities.stream()
                .map(m -> SpplyTargerRtnSearchListResponse.builder()
                        .viewCd("R")
                        .brandId(m.getBrandId() == null ? "" : m.getBrandId())
                        .brandNm(m.getBrandNm() == null ? "" : m.getBrandNm())
                        .rtnPodrUnqNo(m.getRtnPodrUnqNo() == null ? "" : m.getRtnPodrUnqNo())
                        .rtnPodrDtlNo(m.getRtnOdrDtlInfo() == null ? "" : m.getRtnOdrDtlInfo().getRtnPodrDtlNo() == null ? "" : m.getRtnOdrDtlInfo().getRtnPodrDtlNo())
                        .itemId(m.getRtnOdrDtlInfo() == null ? "" : m.getRtnOdrDtlInfo().getItemId() == null ? "" : m.getRtnOdrDtlInfo().getItemId())
                        .itemNm(m.getRtnOdrDtlInfo() == null ? "" : m.getRtnOdrDtlInfo().getItemNm() == null ? "" : m.getRtnOdrDtlInfo().getItemNm())
                        .rtnWhsId(m.getRtnWhsId() == null ? "" : m.getRtnWhsId())
                        .rtnWhsNm(m.getRtnWhsNm() == null ? "" : m.getRtnWhsNm())
                        .rtnLocation(m.getRtnLocation() == null ? "" : m.getRtnLocation())
                        .unitAmt(m.getRtnOdrDtlInfo() == null ? 0 : m.getRtnOdrDtlInfo().getUnitAmt() == null ? 0 : m.getRtnOdrDtlInfo().getUnitAmt())
                        .rtnQty(m.getRtnOdrDtlInfo() == null ? 0 : m.getRtnOdrDtlInfo().getRtnQty() == null ? 0 : m.getRtnOdrDtlInfo().getRtnQty())
                        .rtnAmt(m.getRtnOdrDtlInfo() == null ? 0 : m.getRtnOdrDtlInfo().getRtnAmt() == null ? 0 : m.getRtnOdrDtlInfo().getRtnAmt())
                        .colReqDt(m.getColReqDt() == null ? "" : m.getColReqDt())
                        .build())
                .collect(Collectors.toList());
        return GridResponse.toDTO(gridPage, gridTotal, gridRecords, gridRows);
    }


    /* 공급사 납품서 생성 초기 정보*/
    public SpplyPickInitResponse selectSpplyShmtOdrInfo(String brandId, String spplyId, String rtnPodrUnqNo) throws Exception {
        RtnOdrBaseInfoEntity entity = colOdrMapper.selectSpplyColOdrInfo(brandId, spplyId, rtnPodrUnqNo);
        List<RtnOdrDtlPtclEntity> dtlEntity = colOdrMapper.selectSpplyColRtnList(rtnPodrUnqNo);

        return SpplyPickInitResponse.toDTO(entity, dtlEntity);
    }

    /* 공급사 수거서 생성 저장 */
    @Transactional
    public void saveSpplyPick(String userId, SaveSpplyPickRequest saveSpplyPickRequest) throws Exception {
        try {

            List<RtnOdrDtlPtclEntity> gridInsertData = saveSpplyPickRequest.toCodeEntities(saveSpplyPickRequest.getGridInsertData());
            List<RtnOdrBaseInfoEntity> jobList = colOdrMapper.selectPickJobDataList(saveSpplyPickRequest.getBrandId(), gridInsertData);
            long colTotalAmt = 0;
            int colQty = 0;
            String colPickUnqNo = "";

            PickInfoMgntEntity pickInfoMgntEntity = new PickInfoMgntEntity();
            List<PickDtlPtclEntity> pickDtlList = new ArrayList<PickDtlPtclEntity>();


            for (int i = 0; i < jobList.size(); i++) {
                RtnOdrBaseInfoEntity entity = jobList.get(i);

                for (int j = 0; j < gridInsertData.size(); j++) {
                    RtnOdrDtlPtclEntity reqEntity = gridInsertData.get(j);
                    if (entity.getRtnPodrUnqNo().equals(reqEntity.getRtnPodrUnqNo()) && entity.getRtnOdrDtlInfo().getRtnPodrDtlNo().equals(reqEntity.getRtnPodrDtlNo())) {
                        if (reqEntity.getColQty() > entity.getRtnOdrDtlInfo().getRtnOdrQty()) {
                            // 반품발주수량보다 수거수량이 많을수 없습니다.
                            throw new GlobalException(SystemMessageCode.ERR_POPICKQTY);
                        }
                        colQty = reqEntity.getColQty();
                        break;
                    }
                }

                colTotalAmt = colTotalAmt + (entity.getRtnOdrDtlInfo().getUnitAmt() * colQty);
                if (i == 0 ) {
                    colPickUnqNo =  fnGetDealNo(15, ""); // 수거서번호

                    pickInfoMgntEntity.setColPickUnqNo(colPickUnqNo);
                    pickInfoMgntEntity.setSpplyId(entity.getSpplyId());
                    pickInfoMgntEntity.setSpplyNm(entity.getSpplyNm());
                    pickInfoMgntEntity.setStoreId(entity.getStoreId());
                    pickInfoMgntEntity.setPickMgrNm(saveSpplyPickRequest.getPickMgrNm());
                    pickInfoMgntEntity.setPickMgrTelNo(saveSpplyPickRequest.getPickMgrTelNo().replaceAll("-",""));
                    pickInfoMgntEntity.setPndrTelNo(entity.getPickTelNo().replaceAll("-",""));
                    pickInfoMgntEntity.setPickStCd("PICK0");
                    pickInfoMgntEntity.setColReqDt(saveSpplyPickRequest.getColSchDt().replaceAll("-",""));
                    pickInfoMgntEntity.setColSchDt(saveSpplyPickRequest.getColSchDt().replaceAll("-",""));
                    pickInfoMgntEntity.setColCmpltDt(null);
                    pickInfoMgntEntity.setWhsId(entity.getWhsId());
                    pickInfoMgntEntity.setLocation(entity.getLocation());
                    pickInfoMgntEntity.setColDvsn(entity.getColDvsn());
                    pickInfoMgntEntity.setWhsDvsn(entity.getColDvsn() == "C001" ? "W02" : "W01");  // 본사엣 수거면 매장,,, 공급사에서 수거면 본사
                    pickInfoMgntEntity.setBrandId(entity.getBrandId());
                    pickInfoMgntEntity.setColDesc(null);

                    PickDtlPtclEntity pickDtlEntity = new PickDtlPtclEntity();
                    pickDtlEntity.setColPickUnqNo(colPickUnqNo);

                    pickDtlEntity.setColPickDtlNo("10");
                    pickDtlEntity.setRtnPodrUnqNo(entity.getRtnPodrUnqNo());
                    pickDtlEntity.setRtnPodrDtlNo(entity.getRtnOdrDtlInfo().getRtnPodrDtlNo());
                    pickDtlEntity.setRtnReqUnqNo(entity.getRtnOdrDtlInfo().getRtnReqUnqNo());
                    pickDtlEntity.setRtnReqDtlNo(entity.getRtnOdrDtlInfo().getRtnReqDtlNo());
                    pickDtlEntity.setColCmdUnqNo(null);
                    pickDtlEntity.setColCmdDtlNo("10");
                    pickDtlEntity.setItemId(entity.getRtnOdrDtlInfo().getItemId());
                    pickDtlEntity.setItemNm(entity.getRtnOdrDtlInfo().getItemNm());
                    pickDtlEntity.setUnit(entity.getRtnOdrDtlInfo().getUnit());
                    pickDtlEntity.setUnitAmt(entity.getRtnOdrDtlInfo().getUnitAmt());
                    pickDtlEntity.setRodrQty(entity.getRtnOdrDtlInfo().getRtnOdrQty());
                    pickDtlEntity.setColQty(colQty);
                    colTotalAmt = colQty * entity.getRtnOdrDtlInfo().getUnitAmt();
                    pickDtlEntity.setColAmt(colTotalAmt);

                    pickDtlList.add(pickDtlEntity);
                }

                int dtlNo = (i+1) * 10;

                PickDtlPtclEntity pickDtlEntity = new PickDtlPtclEntity();
                pickDtlEntity.setColPickUnqNo(colPickUnqNo);

                pickDtlEntity.setColPickDtlNo("10");
                pickDtlEntity.setRtnPodrUnqNo(entity.getRtnPodrUnqNo());
                pickDtlEntity.setRtnPodrDtlNo(entity.getRtnOdrDtlInfo().getRtnPodrDtlNo());
                pickDtlEntity.setRtnReqUnqNo(entity.getRtnOdrDtlInfo().getRtnReqUnqNo());
                pickDtlEntity.setRtnReqDtlNo(entity.getRtnOdrDtlInfo().getRtnReqDtlNo());
                pickDtlEntity.setColCmdUnqNo(null);
                pickDtlEntity.setColCmdDtlNo("10");
                pickDtlEntity.setItemId(entity.getRtnOdrDtlInfo().getItemId());
                pickDtlEntity.setItemNm(entity.getRtnOdrDtlInfo().getItemNm());
                pickDtlEntity.setUnit(entity.getRtnOdrDtlInfo().getUnit());
                pickDtlEntity.setUnitAmt(entity.getRtnOdrDtlInfo().getUnitAmt());
                pickDtlEntity.setRodrQty(entity.getRtnOdrDtlInfo().getRtnOdrQty());
                pickDtlEntity.setColQty(colQty);
                pickDtlEntity.setColAmt(colQty * entity.getRtnOdrDtlInfo().getUnitAmt());
                colTotalAmt = colTotalAmt + (colQty * entity.getRtnOdrDtlInfo().getUnitAmt());
                pickDtlList.add(pickDtlEntity);

            }

            pickInfoMgntEntity.setColTotAmt(colTotalAmt);
            colOdrMapper.insertPickMgntBase(userId, pickInfoMgntEntity);
            colOdrMapper.insertPickDtlPtcl(userId, pickDtlList);



            for (int poi = 0; poi < gridInsertData.size(); poi++) {
                gridInsertData.get(poi).setRodrDtlStCd("ROD3");
            }
            rtnOdrMapper.updateChangeMultiRodrDtlStCd(userId, gridInsertData);

            // 반품발주 헤더 상태 처리  updateChgMultiPodrStCd
            String rtnPodrUnqNo = "";
            for (int k = 0; k < gridInsertData.size(); k++) {
                if (!rtnPodrUnqNo.equals(gridInsertData.get(k).getRtnPodrUnqNo())) {
                    rtnPodrUnqNo = gridInsertData.get(k).getRtnPodrUnqNo();
                    String rtnOdrStCd = rtnOdrService.selectRtnOdrDtlStatusList(rtnPodrUnqNo);
                    rtnOdrMapper.updateChgRodrStCd(userId, rtnPodrUnqNo, rtnOdrStCd);
                }
            }

        } catch (GlobalException e) {
            //e.getStackTrace();
            e.printStackTrace();
            throw new GlobalException(e.getSystemMessageCode());
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
    }


    /*
      납품서 현황 그리드 리스트 조회

    public GridResponse selectDeliGridList(String sBrandId, String dtGubun,  String fromDt, String toDt, String sDlvStCd, String sSpplyId, String sWhsId, String sItemNm, GridRequest gridRequest) throws Exception {
        int gridPage = gridRequest.getGridPage();
        int gridSize = gridRequest.getGridSize();

        fromDt = fromDt == null ? null : fromDt.replace(".", "");
        toDt = toDt == null ? null : toDt.replace(".", "");

        int gridRecords = shmtOdrMapper.selectDeliGridCnt(sBrandId, dtGubun, fromDt, toDt, sDlvStCd, sSpplyId, sWhsId, sItemNm);
        int gridTotal = fnCalculateGridTotal(gridSize, gridRecords);
        List<DeliInfoMgntEntity> entities = shmtOdrMapper.selectDeliGridList(sBrandId, dtGubun, fromDt, toDt, sDlvStCd, sSpplyId, sWhsId, sItemNm, gridRequest);
        List<Object> gridRows = entities.stream()
                .map(m -> DeliDtlSearchListResponse.builder()
                        .viewCd("R")
                        .brandId(m.getBrandId() == null ? "" : m.getBrandId())
                        .brandNm(m.getBrandNm() == null ? "" : m.getBrandNm())
                        .dlvSttmtUnqNo(m.getDlvSttmtUnqNo() == null ? "" : m.getDlvSttmtUnqNo())
                        .spplyId(m.getSpplyId() == null ? "" : m.getSpplyId())
                        .spplyNm(m.getSpplyNm() == null ? "" : m.getSpplyNm())
                        .dlvStCd(m.getDlvStCd() == null ? "" : m.getDlvStCd())
                        .dlvStNm(m.getDlvStNm() == null ? "" : m.getDlvStNm())
                        .whsId(m.getWhsId() == null ? "" :m.getWhsId())
                        .location(m.getLocation() == null ? "" : m.getLocation())
                        .whsNm(m.getWhsNm() == null ? "" : m.getWhsNm() + " - " + m.getLocationNm())
                        .podrRegDt(m.getPodrRegDt() == null ? "" : m.getPodrRegDt())
                        .dlvRegDt(m.getDlvRegDt() == null ? "" : m.getDlvRegDt())
                        .dlvReqDt(m.getDlvReqDt() == null ? "" : m.getDlvReqDt())
                        .dlvSchDt(m.getDlvSchDt() == null ? "" : m.getDlvSchDt())
                        .dlvCmpltDt(m.getDlvCmpltDt() == null ? "" : m.getDlvCmpltDt())
                        .itemId(m.getDeliDtlMgntInfo() == null ? "" : m.getDeliDtlMgntInfo().getItemId() == null ? "" : m.getDeliDtlMgntInfo().getItemId())
                        .itemNm(m.getDeliDtlMgntInfo() == null ? "" : m.getDeliDtlMgntInfo().getItemNm() == null ? "" : m.getDeliDtlMgntInfo().getItemNm())
                        .podrQty(m.getDeliDtlMgntInfo() == null ? 0 : m.getDeliDtlMgntInfo().getPodrQty() == null ? 0 : m.getDeliDtlMgntInfo().getPodrQty())
                        .dlvQty(m.getDeliDtlMgntInfo() == null ? 0 : m.getDeliDtlMgntInfo().getDlvQty() == null ? 0 : m.getDeliDtlMgntInfo().getDlvQty())
                        .unit(m.getDeliDtlMgntInfo() == null ? "" : m.getDeliDtlMgntInfo().getUnit() == null ? "" : m.getDeliDtlMgntInfo().getUnit())
                        .unitAmt(m.getDeliDtlMgntInfo() == null ? 0 : m.getDeliDtlMgntInfo().getUnitAmt() == null ? 0 : m.getDeliDtlMgntInfo().getUnitAmt())
                        .dlvMgrNm(m.getDlvMgrNm() == null ? "" : m.getDlvMgrNm())
                        .dlvMgrTelNo(m.getDlvMgrTelNo() == null ? "" : m.getDlvMgrTelNo())
                        .build())
                .collect(Collectors.toList());
        return GridResponse.toDTO(gridPage, gridTotal, gridRecords, gridRows);
    }
*/

    /*
      공급사  - 수거서 현황 그리드 리스트 조회
 */
    public GridResponse selectSpplyPickGridList(String sBrandId, String fromDt, String toDt, String sSpplyId, String sWhsId, String sItemNm, GridRequest gridRequest) throws Exception {
        int gridPage = gridRequest.getGridPage();
        int gridSize = gridRequest.getGridSize();

        fromDt = fromDt == null ? null : fromDt.replace(".", "");
        toDt = toDt == null ? null : toDt.replace(".", "");

        int gridRecords = colOdrMapper.selectColSpplyPickGridCnt(sBrandId,  fromDt, toDt, sSpplyId, sWhsId, sItemNm);
        int gridTotal = fnCalculateGridTotal(gridSize, gridRecords);
        List<PickInfoMgntEntity> entities = colOdrMapper.selectColSpplyPickGridList(sBrandId, fromDt, toDt, sSpplyId, sWhsId, sItemNm, gridRequest);
        List<Object> gridRows = entities.stream()
                .map(m -> PickDtlSearchListResponse.builder()
                        .viewCd("R")
                        .brandId(m.getBrandId() == null ? "" : m.getBrandId())
                        .brandNm(m.getBrandNm() == null ? "" : m.getBrandNm())
                        .colPickUnqNo(m.getColPickUnqNo() == null ? "" : m.getColPickUnqNo())
                        .pickStCd(m.getPickStCd() == null ? "" : m.getPickStCd())
                        .pickStNm(m.getPickStNm() == null ? "" : m.getPickStNm())
                        .colRegDt(m.getColRegDt() == null ? "" : m.getColRegDt())
                        .colReqDt(m.getColReqDt() == null ? "" : m.getColReqDt())
                        .colSchDt(m.getColSchDt() == null ? "" : m.getColSchDt())
                        .colCmpltDt(m.getColCmpltDt() == null ? "" : m.getColCmpltDt())
                        .itemId(m.getPickDtlInfo() == null ? "" : m.getPickDtlInfo().getItemId() == null ? "" :  m.getPickDtlInfo().getItemId())
                        .itemNm(m.getPickDtlInfo() == null ? "" : m.getPickDtlInfo().getItemNm() == null ? "" :  m.getPickDtlInfo().getItemNm())
                        .colQty(m.getPickDtlInfo() == null ? 0 : m.getPickDtlInfo().getColQty() == null ? 0 :  m.getPickDtlInfo().getColQty())
                        .rtnRsnDvsn(m.getPickDtlInfo() == null ? "" : m.getPickDtlInfo().getRtnRsnDvsn() == null ? "" :  m.getPickDtlInfo().getRtnRsnDvsn())
                        .rtnRsnDvsnNm(m.getPickDtlInfo() == null ? "" : m.getPickDtlInfo().getRtnRsnDvsnNm() == null ? "" :  m.getPickDtlInfo().getRtnRsnDvsnNm())
                        .unitAmt(m.getPickDtlInfo() == null ? 0 : m.getPickDtlInfo().getUnitAmt() == null ? 0 :  m.getPickDtlInfo().getUnitAmt())
                        .colAmt(m.getPickDtlInfo() == null ? 0 : m.getPickDtlInfo().getColAmt() == null ? 0 :  m.getPickDtlInfo().getColAmt())
                        .whsNm(m.getWhsNm() == null ? "" : m.getWhsNm() + " - " + m.getLocationNm())
                        .pickMgrNm(m.getPickMgrNm() == null ? "" : m.getPickMgrNm())
                        .pickMgrTelNo(m.getPickMgrTelNo() == null ? "" : m.getPickMgrTelNo())
                        .build())
                .collect(Collectors.toList());
        return GridResponse.toDTO(gridPage, gridTotal, gridRecords, gridRows);
    }

    /*
      반품처리 그리드 리스트 조회
 */
    public GridResponse selectRtnStockTargetList(String sBrandId, String sStoreId,  String fromDt, String toDt, String sRtnWhsId, String sSpplyId, String sWhsId, String sItemNm, GridRequest gridRequest) throws Exception {
        int gridPage = gridRequest.getGridPage();
        int gridSize = gridRequest.getGridSize();

        fromDt = fromDt == null ? null : fromDt.replace(".", "");
        toDt = toDt == null ? null : toDt.replace(".", "");

        int gridRecords = colOdrMapper.selectColRtnInstGridCnt(sBrandId, sStoreId, fromDt, toDt, sSpplyId, sRtnWhsId,  sWhsId,  sItemNm);
        int gridTotal = fnCalculateGridTotal(gridSize, gridRecords);
        List<PickInfoMgntEntity> entities = colOdrMapper.selectColRtnInstGridList(sBrandId, sStoreId, fromDt, toDt, sSpplyId, sRtnWhsId, sWhsId, sItemNm, gridRequest);
        List<Object> gridRows = entities.stream()
                .map(m -> RtnStockTargetResponse.builder()
                        .viewCd("R")
                        .brandId(m.getBrandId() == null ? "" : m.getBrandId())
                        .brandNm(m.getBrandNm() == null ? "" : m.getBrandNm())
                        .colPickUnqNo(m.getColPickUnqNo() == null ? "" : m.getColPickUnqNo())
                        .colPickDtlNo(m.getPickDtlInfo() == null ? "" : m.getPickDtlInfo().getColPickUnqNo() == null ? "" :m.getPickDtlInfo().getColPickUnqNo() )
                        .rtnPodrUnqNo(m.getPickDtlInfo() == null ? "" : m.getPickDtlInfo().getRtnPodrUnqNo() == null ? "" :m.getPickDtlInfo().getRtnPodrUnqNo() )
                        .rtnPodrDtlNo(m.getPickDtlInfo() == null ? "" : m.getPickDtlInfo().getRtnPodrDtlNo() == null ? "" :m.getPickDtlInfo().getRtnPodrDtlNo() )
                        .itemId(m.getPickDtlInfo() == null ? "" : m.getPickDtlInfo().getItemId() == null ? "" :m.getPickDtlInfo().getItemId() )
                        .itemNm(m.getPickDtlInfo() == null ? "" : m.getPickDtlInfo().getItemNm() == null ? "" :m.getPickDtlInfo().getItemNm() )
                        .colQty(m.getPickDtlInfo() == null ? 0 : m.getPickDtlInfo().getColQty() == null ? 0 :m.getPickDtlInfo().getColQty() )
                        .colReqQty(m.getPickDtlInfo() == null ? 0 : m.getPickDtlInfo().getColReqQty() == null ? 0 :m.getPickDtlInfo().getColReqQty() )
                        .colAmt(m.getPickDtlInfo() == null ? 0 : m.getPickDtlInfo().getColAmt() == null ? 0 :m.getPickDtlInfo().getColAmt() )
                        .whsId(m.getWhsId() == null ? "" : m.getWhsId())
                        .location(m.getLocation() == null ? "" : m.getLocation())
                        .whsNm(m.getWhsNm() == null ? "" : m.getWhsNm())
                        .rtnWhsId(m.getRtnWhsId() == null ? "" : m.getRtnWhsId())
                        .rtnLocation(m.getRtnLocation() == null ? "" : m.getRtnLocation())
                        .rtnWhsNm(m.getRtnWhsNm() == null ? "" : m.getRtnWhsNm())
                        .colSchDt(m.getColSchDt() == null ? "" : m.getColSchDt())
                        .pickMgrNm(m.getPickMgrNm() == null ? "" : m.getPickMgrNm())
                        .pickMgrTelNo(m.getPickMgrTelNo() == null ? "" : m.getPickMgrTelNo())
                        .spplyId(m.getSpplyId() == null ? "" : m.getSpplyId())
                        .spplyNm(m.getSpplyNm() == null ? "" : m.getSpplyNm())
                        .build())
                .collect(Collectors.toList());
        return GridResponse.toDTO(gridPage, gridTotal, gridRecords, gridRows);
    }

    /* 입고처리 저장
    @Transactional
    public void saveRtnStockProc(String userId, SaveRtnStockProcRequest saveRtnStockProcRequest) throws Exception {
        try {

            List<RtnProcInfoPtclEntity> gridInsertData = saveRtnStockProcRequest.toCodeEntities(saveRtnStockProcRequest.getGridInsertData());
            List<RtnProcInfoPtclEntity> serarchList = new ArrayList<RtnProcInfoPtclEntity>();
            if (gridInsertData.size() > 0) {
                for (int i = 0 ; i < gridInsertData.size() ; i++) {
                    RtnProcInfoPtclEntity whsProcData = gridInsertData.get(i);
                    DeliDtlMgntEntity deliData = new DeliDtlMgntEntity();
                    deliData.setDlvSttmtDtlNo(whsProcData.getDlvSttmtUnqNo() + whsProcData.getDlvSttmtDtlNo());
                    serarchList.add(deliData);
                }

                List<DeliInfoMgntEntity> targetList  =   shmtOdrMapper.selectWarehouseProcList(saveWarehouseProcRequest.getBrandId(), saveWarehouseProcRequest.getWhsId(), serarchList);

                List<WhsProcInfoPtclEntity> gridUpdateData = new ArrayList<WhsProcInfoPtclEntity>();
                List<StckMgntBaseInfoEntity> wareGridUpdateData = new ArrayList<StckMgntBaseInfoEntity>();
                List<DeliDtlMgntEntity> deliDtlGridUpdateData = new ArrayList<DeliDtlMgntEntity>();
                List<ShmtOdrDtlEntity> shmtDtlGridUpdateData = new ArrayList<ShmtOdrDtlEntity>();
                List<PchOdrDtlPtclEntity> PchOdrDtlGridUpdateData = new ArrayList<PchOdrDtlPtclEntity>();


                // loan_mgnt_unq_no 매장납품일 경우
                //selectStoreLoanMgntUnqNo
                String loanMgntUnqNo = "";
                long loanAmt = 0 ; // 여신차감할 금액 (사용금액 - 여신차감금액,   잔액금액 - 여신차감금액)
                if (!"".equals(saveWarehouseProcRequest.getStoreId())) {
                    loanMgntUnqNo = storeBaseInfoMapper.selectStoreLoanMgntUnqNo(saveWarehouseProcRequest.getStoreId());
                }

                // 우선 입고수량과 납품수량 비교를 처리함
                for (int i = 0 ; i < gridInsertData.size() ; i++) {
                    WhsProcInfoPtclEntity whsProcData = gridInsertData.get(i);

                    for (int j = 0  ; j < targetList.size() ; j++) {
                        DeliInfoMgntEntity targetData = targetList.get(j);
                        //System.out.println("=whsProcData=======getDlvSttmtUnqNo============>" + i + "==" + j + "==>" + whsProcData.getDlvSttmtUnqNo() +  "==" +  targetData.getDlvSttmtUnqNo());
                       // System.out.println("=whsProcData=======getDlvSttmtDtlNo============>" + i + "==" + j + "==>"  + whsProcData.getDlvSttmtDtlNo() +  "==" +   targetData.getDeliDtlMgntInfo().getDlvSttmtDtlNo());
                        if (whsProcData.getDlvSttmtUnqNo().equals(targetData.getDlvSttmtUnqNo()) && whsProcData.getDlvSttmtDtlNo().equals(targetData.getDeliDtlMgntInfo().getDlvSttmtDtlNo())) {

                            if (whsProcData.getWhsQty() > targetData.getDeliDtlMgntInfo().getDlvQty()) {
                                 // 오류 처리
                                throw new GlobalException(SystemMessageCode.ERR_NOTWAREHOUSE);
                            }

                            // 입고번호 생성
                            whsProcData.setWhsMgntUnqNo(fnGetDealNo(17,""));

                            //System.out.println("=whsProcData=======setWhsMgntUnqNo============>" + i + "==" + j + "==>"  + whsProcData.getWhsMgntUnqNo());

                            whsProcData.setDlvSttmtDtlNo(targetData.getDeliDtlMgntInfo().getDlvSttmtDtlNo());
                            whsProcData.setDlvSttmtUnqNo(targetData.getDlvSttmtUnqNo());
                            whsProcData.setBrandId(targetData.getBrandId());
                            whsProcData.setStoreId(targetData.getStoreId());
                            whsProcData.setWhsId(targetData.getWhsId());
                            whsProcData.setLocation(targetData.getLocation());
                            whsProcData.setSpplyId(targetData.getSpplyId());
                            whsProcData.setSpplyNm(targetData.getSpplyNm());
                            whsProcData.setWhsDvsn(targetData.getWhsDvsn());
                            whsProcData.setPodrQty(targetData.getDeliDtlMgntInfo().getPodrQty());
                            whsProcData.setDlvQty(targetData.getDeliDtlMgntInfo().getDlvQty());
                            whsProcData.setUnit(targetData.getDeliDtlMgntInfo().getUnit());
                            whsProcData.setUnitAmt(targetData.getDeliDtlMgntInfo().getUnitAmt());
                            whsProcData.setDlvReqDt(targetData.getDlvReqDt());
                            whsProcData.setDlvSchDt(targetData.getDlvSchDt());
                            whsProcData.setWhsCnfmNm(userId);
                            whsProcData.setDlvMgrNm(targetData.getDlvMgrNm());
                            whsProcData.setDlvMgrTelNo(targetData.getDlvMgrTelNo());
                            whsProcData.setDlvMgrEmal(targetData.getDlvMgrEmal());
                            whsProcData.setDlvMgrMblNo(targetData.getDlvMgrMblNo());
                            whsProcData.setItemId(targetData.getDeliDtlMgntInfo().getItemId());
                            whsProcData.setItemNm(targetData.getDeliDtlMgntInfo().getItemNm());
                            whsProcData.setWhsAmt(Long.valueOf(targetData.getDeliDtlMgntInfo().getUnitAmt() * whsProcData.getWhsQty()));
                            whsProcData.setWhsDlvChrg(null);
                            //whsProcData.setDlvMgrId(targetData.getdlvmgr);
                            whsProcData.setDlvCnfmDocFileNo(whsProcData.getDlvCnfmDocFileNo());
                            whsProcData.setSttlYn("N");
                            whsProcData.setPchPodrUnqNo(targetData.getDeliDtlMgntInfo().getPchPodrUnqNo());
                            whsProcData.setPchPodrDtlNo(targetData.getDeliDtlMgntInfo().getPchPodrDtlNo());
                            whsProcData.setPchReqUnqNo(targetData.getDeliDtlMgntInfo().getPchReqUnqNo());
                            whsProcData.setPchReqDtlNo(targetData.getDeliDtlMgntInfo().getPchReqDtlNo());
                            whsProcData.setShmtReqUnqNo(targetData.getDeliDtlMgntInfo().getShmtReqUnqNo());
                            whsProcData.setShmtReqDtlNo(targetData.getDeliDtlMgntInfo().getShmtReqDtlNo());

                            gridUpdateData.add(whsProcData);

                            // 정산할때 함 여신관련한 여신차감할 입고금액 세팅
                            //loanAmt = loanAmt + (targetData.getDeliDtlMgntInfo().getUnitAmt() * whsProcData.getWhsQty());

                            // 재고처리를 위한 작업업
                            StckMgntBaseInfoEntity smbaseInfoEntity = new StckMgntBaseInfoEntity();
                            smbaseInfoEntity.setBrandId(saveWarehouseProcRequest.getBrandId());
                            smbaseInfoEntity.setWhsId(saveWarehouseProcRequest.getWhsId());
                            smbaseInfoEntity.setLocation(whsProcData.getLocation());
                            smbaseInfoEntity.setItemId(whsProcData.getItemId());
                            smbaseInfoEntity.setStckQty(whsProcData.getWhsQty());
                            wareGridUpdateData.add(smbaseInfoEntity);

                            // 납품서 상태 정의  dlvSttmtUnqNo  dlvDtlStCd
                            DeliDtlMgntEntity deliDtlEntity = new DeliDtlMgntEntity();
                            deliDtlEntity.setDlvSttmtUnqNo(whsProcData.getDlvSttmtUnqNo());
                            deliDtlEntity.setDlvSttmtDtlNo(whsProcData.getDlvSttmtDtlNo());
                            if (whsProcData.getWhsQty() > 0 ) {
                                deliDtlEntity.setDlvDtlStCd("DLD1");
                            } else {
                                deliDtlEntity.setDlvDtlStCd("DLD3");
                            }
                            deliDtlGridUpdateData.add(deliDtlEntity);


                            // 출하지시 상태정의
                            if (!"".equals(whsProcData.getShmtReqUnqNo()) && whsProcData.getShmtReqUnqNo() != null) {
                                ShmtOdrDtlEntity shmtDtlData = new ShmtOdrDtlEntity();
                                shmtDtlData.setShmtReqUnqNo(whsProcData.getShmtReqUnqNo());
                                shmtDtlData.setShmtReqDtlNo(whsProcData.getPchReqDtlNo());
                                if (whsProcData.getWhsQty() > 0 ) {
                                    shmtDtlData.setShmtDtlStCd("SHMD2");
                                } else {
                                    shmtDtlData.setShmtDtlStCd("SHMD3");
                                }

                                shmtDtlGridUpdateData.add(shmtDtlData);
                            }

                            PchOdrDtlPtclEntity pchOdrDtlData = new PchOdrDtlPtclEntity();
                            pchOdrDtlData.setPchPodrUnqNo(whsProcData.getPchPodrUnqNo());
                            pchOdrDtlData.setPchPodrDtlNo(whsProcData.getPchPodrDtlNo());
                            if (whsProcData.getWhsQty() > 0 ) {
                                pchOdrDtlData.setPodrDtlStCd("POD4");
                            } else {
                                pchOdrDtlData.setPodrDtlStCd("POD2");

                                if (!"".equals(loanMgntUnqNo)) {
                                    loanAmt = loanAmt + ( targetData.getDeliDtlMgntInfo().getDlvQty() * targetData.getDeliDtlMgntInfo().getUnitAmt() ) ;
                                }
                            }

                            PchOdrDtlGridUpdateData.add(pchOdrDtlData);
                            break;
                        }


                    }  // or문 종료
                }

                    // 입고데이타 저장
                    shmtOdrMapper.insertWhsProcInfoPtcl(userId, gridUpdateData);

                    // 재고관련 정리 필요
                    stockMngMapper.updateWarehouseStockQty(userId, wareGridUpdateData);

                    // 납품서 상세 처리
                    shmtOdrMapper.updateDeliDtlStCd(userId, deliDtlGridUpdateData);

                    // 납품서 헤더 상태 처리
                    List<DeliInfoMgntEntity> deliInfoStList = shmtOdrMapper.selectDeliInfoStList(userId, deliDtlGridUpdateData);
                    shmtOdrMapper.updateDeliInfoStCd(userId, deliInfoStList);

                    // 출하지시서 상태 업데이트
                    if (shmtDtlGridUpdateData.size() > 0) {
                        shmtOdrMapper.updateShmtDtlStCd(userId,shmtDtlGridUpdateData );
                        List<ShmtOdrBaseEntity> shmtInfoStList = shmtOdrMapper.selectShmtInfoStList(userId, shmtDtlGridUpdateData);
                        shmtOdrMapper.updateShmtInfoStCd(userId, shmtInfoStList);
                    }


                    shmtOdrMapper.updatePchPodrDtlStCd(userId, PchOdrDtlGridUpdateData);

                    String pchPodrUnqNo = "";
                    for (int k = 0; k < PchOdrDtlGridUpdateData.size(); k++) {
                        if (!pchPodrUnqNo.equals(PchOdrDtlGridUpdateData.get(k).getPchPodrUnqNo())) {
                            pchPodrUnqNo = PchOdrDtlGridUpdateData.get(k).getPchPodrUnqNo();
                            String pchOdrStCd = pchOrdService.selectOdrDtlStatusList(pchPodrUnqNo);
                            pchOrdMapper.updateChgPodrStCd(userId, pchOdrStCd, pchPodrUnqNo);
                        }
                    }

                    // 발주 헤더가 취소면 해당 발주건에 대한 사용금액을 원복 처리한다.
                    if (!"".equals(loanMgntUnqNo) && loanAmt > 0) {
                        // 여신관련 정리 필요 loanAmt 업데이트
                        loanStateMapper.updateloanMinusInstProc(loanMgntUnqNo, saveWarehouseProcRequest.getStoreId(), loanAmt);
                    }



            }

        } catch (GlobalException e) {
            //e.getStackTrace();
            e.printStackTrace();
            throw new GlobalException(e.getSystemMessageCode());
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
    }
*/

    /*
      입고 현황 그리드 리스트 조회  @Param("sBrandId") String sBrandId, @Param("sStoreId") String sStoreId,
      @Param("fromDt") String fromDt, @Param("toDt") String toDt, @Param("sPchPodrUnqNo") String sPchPodrUnqNo, @Param("sSpplyId") String sSpplyId, @Param("sWhsId") String sWhsId, @Param("sItemNm") String sItemNm

    public GridResponse selectWarehouseGridList(String sBrandId, String sStoreId,  String fromDt, String toDt, String sPchPodrUnqNo, String sSpplyId, String sWhsId, String sItemNm, GridRequest gridRequest) throws Exception {
        int gridPage = gridRequest.getGridPage();
        int gridSize = gridRequest.getGridSize();

        fromDt = fromDt == null ? null : fromDt.replace(".", "");
        toDt = toDt == null ? null : toDt.replace(".", "");

        int gridRecords = shmtOdrMapper.selectWarehouseCnt(sBrandId, sStoreId, fromDt, toDt, sPchPodrUnqNo, sSpplyId, sWhsId, sItemNm);
        int gridTotal = fnCalculateGridTotal(gridSize, gridRecords);
        List<WhsProcInfoPtclEntity> entities = shmtOdrMapper.selectWarehouseList(sBrandId, sStoreId, fromDt, toDt, sPchPodrUnqNo, sSpplyId, sWhsId, sItemNm, gridRequest);
        List<Object> gridRows = entities.stream()
                .map(m -> WarehouseSearchGridListResponse.builder()
                        .viewCd("R")
                        .pchPodrUnqNo(m.getPchPodrUnqNo() == null ? "" : m.getPchPodrUnqNo())
                        .pchPodrDtlNo(m.getPchPodrDtlNo() == null ? "" : m.getPchPodrDtlNo())
                        .dlvSttmtUnqNo(m.getDlvSttmtUnqNo() == null ? "" : m.getDlvSttmtUnqNo())
                        .dlvSttmtDtlNo(m.getDlvSttmtDtlNo() == null ? "" : m.getDlvSttmtDtlNo() )
                        .podrQty(m.getPodrQty() == null ? 0 : m.getPodrQty())
                        .dlvQty(m.getDlvQty() == null ? 0 : m.getDlvQty())
                        .whsQty(m.getWhsQty() == null ? 0 : m.getWhsQty())
                        .dlvReqDt(m.getDlvReqDt() == null ? "" : m.getDlvReqDt())
                        .dlvSchDt(m.getDlvSchDt() == null ? "" : m.getDlvSchDt())
                        .dlvCmpltDt(m.getDlvCmpltDt() == null ? "" : m.getDlvCmpltDt())
                        .whsCnfmNm(m.getWhsCnfmNm() == null ? "" : m.getWhsCnfmNm())
                        .dlvMgrNm(m.getDlvMgrNm() == null ? "" : m.getDlvMgrNm())
                        .dlvMgrTelNo(m.getDlvMgrTelNo() == null ? "" : m.getDlvMgrTelNo())
                        .itemId(m.getItemId() == null ? "" : m.getItemId())
                        .itemNm(m.getItemNm() == null ? "" : m.getItemNm())
                        .unit(m.getUnit() == null ? "" : m.getUnit())
                        .whsAmt(m.getWhsAmt() == null ? 0 : m.getWhsAmt())
                        .build())
                .collect(Collectors.toList());
        return GridResponse.toDTO(gridPage, gridTotal, gridRecords, gridRows);
    }

 */

}



